@@ -1,6 +1,6 @@
 /* trap.h -- data structures used in the trap mechanism. */
 
-/* Copyright (C) 1993-2010 Free Software Foundation, Inc.
+/* Copyright (C) 1993-2013 Free Software Foundation, Inc.
 
    This file is part of GNU Bash, the Bourne Again SHell.
 
@@ -46,7 +46,7 @@
 #define BASH_NSIG	NSIG+3
 
 /* Flags values for decode_signal() */
-#define DSIG_SIGPREFIX	0x01		/* don't alllow `SIG' PREFIX */
+#define DSIG_SIGPREFIX	0x01		/* don't allow `SIG' PREFIX */
 #define DSIG_NOCASE	0x02		/* case-insensitive comparison */
 
 /* A value which can never be the target of a trap handler. */
@@ -60,11 +60,17 @@
 
 extern char *trap_list[];
 
+extern int trapped_signal_received;
+extern int wait_signal_received;
+extern int running_trap;
+extern int trap_saved_exit_value;
+
 /* Externally-visible functions declared in trap.c. */
 extern void initialize_traps __P((void));
 
 extern void run_pending_traps __P((void));
 
+extern void queue_sigchld_trap __P((int));
 extern void maybe_set_sigchld_trap __P((char *));
 extern void set_impossible_sigchld_trap __P((void));
 extern void set_sigchld_trap __P((char *));
@@ -73,6 +79,10 @@ extern void set_debug_trap __P((char *));
 extern void set_error_trap __P((char *));
 extern void set_return_trap __P((char *));
 
+extern void maybe_set_debug_trap __P((char *));
+extern void maybe_set_error_trap __P((char *));
+extern void maybe_set_return_trap __P((char *));
+
 extern void set_sigint_trap __P((char *));
 extern void set_signal __P((int, char *));
 
@@ -88,18 +98,26 @@ extern void free_trap_strings __P((void));
 extern void reset_signal_handlers __P((void));
 extern void restore_original_signals __P((void));
 
+extern void get_original_signal __P((int));
 extern void get_all_original_signals __P((void));
 
 extern char *signal_name __P((int));
 
 extern int decode_signal __P((char *, int));
-extern void run_interrupt_trap __P((void));
+extern void run_interrupt_trap __P((int));
 extern int maybe_call_trap_handler __P((int));
 extern int signal_is_special __P((int));
 extern int signal_is_trapped __P((int));
+extern int signal_is_pending __P((int));
 extern int signal_is_ignored __P((int));
 extern int signal_is_hard_ignored __P((int));
+extern void set_signal_hard_ignored __P((int));
 extern void set_signal_ignored __P((int));
 extern int signal_in_progress __P((int));
 
+extern int first_pending_trap __P((void));
+extern int any_signals_trapped __P((void));
+extern void check_signals __P((void));
+extern void check_signals_and_traps __P((void));
+
 #endif /* _TRAP_H_ */
